﻿using CodeFirstTPH.Models;
using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CodeFirstTPH
{
    public class BooksDbContext : DbContext
    {
        public BooksDbContext() : base("BooksConnectionString") { }

        public DbSet<Book> Books { get; set; }

        /*
        Table per Hierarchy (TPH): Enable polymorphism by denormalizing the SQL schema, and utilize a type discriminator 
        column that holds type information 
        This mapping strategy is a winner in terms of both performance and simplicity. 
        It’s the best-performing way to represent polymorphism—both polymorphic and nonpolymorphic queries perform well—and 
        it’s even easy to implement by hand. Ad-hoc reporting is possible without complex joins or unions. 
        Schema evolution is straightforward.
        */
        protected override void OnModelCreating(DbModelBuilder modelBuilder)
        {
            modelBuilder.Entity<Book>()
                .Map<FictionBook>(m => m.Requires("Discriminator").HasValue(0))
                        .Map<NonFictionBook>(m => m.Requires("Discriminator").HasValue(1));
        }
    }
}
